from collections import defaultdict

from phrase_translator.phrase_translator import DictionarySource
from phrase_translator.types import Language, Translation


class FileDictionarySource(DictionarySource):

    SEPARATOR: str = " -> "
    COMMENT: str = "#"
    LANGUAGE_INDICATOR = "lang: "

    def __init__(self, dictionary_files: [str]) -> None:
        self.__dictionary_files: [str] = dictionary_files
        self.__translations: {str: [Translation]} = defaultdict(list)

        self.__load_files()

    def __raise_syntax_error(self, index: int, line: str):
        raise SyntaxError("Malformed Translation on line " + str(index) + ": " + line)

    def __load_files(self) -> None:
        for dictionary_path in self.__dictionary_files:
            with open(dictionary_path, "r") as dictionary_file:
                source_language = Language.UNKNOWN
                target_language = Language.UNKNOWN

                for index, line in enumerate(dictionary_file):
                    if line.startswith(self.COMMENT):
                        continue

                    if line.startswith(self.LANGUAGE_INDICATOR):
                        lang_splits = line[len(self.LANGUAGE_INDICATOR) :].split(
                            self.SEPARATOR
                        )

                        if len(lang_splits) != 2:
                            self.__raise_syntax_error(index, line)

                        source_language = Language(lang_splits[0])
                        target_language = Language(lang_splits[1])

                    line = line.replace("\n", "")
                    splits = line.split(self.SEPARATOR)

                    if len(splits) != 2:
                        self.__raise_syntax_error(index, line)

                    self.__translations[splits[0]].append(
                        Translation(
                            splits[0], splits[1], source_language, target_language
                        )
                    )

    def _provide_translations(self, phrase: str) -> [Translation]:
        return self.__translations[phrase]



from abc import ABC, abstractmethod

from phrase_translator.types import Language, Translation


class DictionarySource(ABC):
    def get_translations(
        self,
        phrase: str,
        source_language: Language = None,
        target_language: Language = None,
    ) -> [Translation]:
        results = []

        for translation in self._provide_translations(phrase):
            if source_language and translation.get_source_lang() != source_language:
                continue

            if target_language and translation.get_target_lang() != target_language:
                continue

            results.append(translation)

        return results

    @abstractmethod
    def _provide_translations(self, phrase: str) -> [Translation]:
        pass


class PhraseTranslator:
    def __init__(
        self, source_language: Language = None, target_language: Language = None
    ) -> None:
        self.__dictionary_sources: [DictionarySource] = []

        self.__source_language = source_language
        self.__target_language = target_language

    def add_dictionary_source(self, dictionary_source: DictionarySource) -> None:
        self.__dictionary_sources.append(dictionary_source)

    def translate_phrase(self, phrase: str) -> [Translation]:
        results = []

        for dictionary_source in self.__dictionary_sources:
            results += dictionary_source.get_translations(
                phrase, self.__source_language, self.__target_language
            )

        return results

    def translate_phrases(self, phrases: [str]) -> [[Translation]]:
        results = [[]]

        for phrase in phrases:
            results.append(self.translate_phrase(phrase))

        return results



import json
from copy import copy
from enum import Enum


class Language(Enum):
    ENGLISH = ("en",)
    GERMAN = ("ge",)
    UNKNOWN = "unknown"


class Translation:
    def __init__(
        self,
        source_phrase: str,
        target_phrase: str,
        source_lang: Language,
        target_lang: Language,
    ) -> None:
        self.__source_phrase = source_phrase
        self.__target_phrase = target_phrase
        self.__source_lang = source_lang
        self.__target_lang = target_lang

    def __str__(self) -> str:
        pass

    def __repr__(self) -> str:
        return self.to_json_string()

    def get_source_lang(self) -> Language:
        return self.__source_lang

    def get_target_lang(self) -> Language:
        return self.__target_lang

    def to_json_string(self) -> str:
        pass
